﻿
Public Class RelayCommand
    Implements ICommand

    Private ReadOnly _execute As Action

    Private ReadOnly _canExecute As Func(Of Boolean)

    'Public Event CanExecuteChanged As EventHandler Implements ICommand.CanExecuteChanged

    Public Custom Event CanExecuteChanged As EventHandler Implements ICommand.CanExecuteChanged

        AddHandler(value As EventHandler)
            If _canExecute IsNot Nothing Then
                AddHandler CommandManager.RequerySuggested, value
            End If
        End AddHandler
        RemoveHandler(value As EventHandler)
            If _canExecute IsNot Nothing Then
                RemoveHandler CommandManager.RequerySuggested, value
            End If
        End RemoveHandler

        RaiseEvent(sender As Object, e As EventArgs)
            'This is the RaiseEvent block
            CommandManager.InvalidateRequerySuggested()
        End RaiseEvent
    End Event

    'Public Sub RaiseCanExecuteChanged()
    '    RaiseEvent CanExecuteChanged(Me, New EventArgs())
    'End Sub

    Public Sub New(execute As Action)
        Me.New(execute, Nothing)
    End Sub

    Public Sub New(execute As Action, canExecute As Func(Of Boolean))
        If execute Is Nothing Then
            Throw New ArgumentNullException(NameOf(execute))
        End If

        Me._execute = execute
        Me._canExecute = canExecute
    End Sub

    Public Function CanExecute(parameter As Object) As Boolean Implements ICommand.CanExecute
        If _canExecute Is Nothing Then
            Return True
        Else
            Return _canExecute()
        End If
    End Function

    Public Sub Execute(parameter As Object) Implements ICommand.Execute
        _execute()
    End Sub

    Public Sub OnCanExecuteChanged()
        RaiseEvent CanExecuteChanged(Me, EventArgs.Empty)
    End Sub

End Class

Public Class RelayCommand(Of T)
    Implements ICommand

    Private ReadOnly _execute As Action(Of T)

    Private ReadOnly _canExecute As Func(Of T, Boolean)

    'Public Event CanExecuteChanged As EventHandler Implements ICommand.CanExecuteChanged

    Public Custom Event CanExecuteChanged As EventHandler Implements ICommand.CanExecuteChanged

        AddHandler(value As EventHandler)
            If _canExecute IsNot Nothing Then
                AddHandler CommandManager.RequerySuggested, value
            End If
        End AddHandler
        RemoveHandler(value As EventHandler)
            If _canExecute IsNot Nothing Then
                RemoveHandler CommandManager.RequerySuggested, value
            End If
        End RemoveHandler

        RaiseEvent(sender As Object, e As EventArgs)
            'This is the RaiseEvent block
            CommandManager.InvalidateRequerySuggested()
        End RaiseEvent
    End Event

    'Public Sub RaiseCanExecuteChanged()
    '    RaiseEvent CanExecuteChanged(Me, New EventArgs())
    'End Sub

    Public Sub New(execute As Action(Of T))
        Me.New(execute, Nothing)
    End Sub

    Public Sub New(execute As Action(Of T), canExecute As Func(Of T, Boolean))
        If execute Is Nothing Then
            Throw New ArgumentNullException(NameOf(execute))
        End If

        Me._execute = execute
        Me._canExecute = canExecute
    End Sub

    Public Function CanExecute(parameter As Object) As Boolean Implements ICommand.CanExecute
        If _canExecute Is Nothing Then
            Return True
        Else
            Return _canExecute(CType(parameter, T))
        End If
    End Function

    Public Sub Execute(parameter As Object) Implements ICommand.Execute
        _execute(CType(parameter, T))
    End Sub

    Public Sub OnCanExecuteChanged()
        RaiseEvent CanExecuteChanged(Me, EventArgs.Empty)
    End Sub

End Class
